package com.telecomscloud.api;

import com.telecomscloud.ApiCallback;
import com.telecomscloud.ApiClient;
import com.telecomscloud.ApiException;
import com.telecomscloud.ApiResponse;
import com.telecomscloud.Configuration;
import com.telecomscloud.Pair;
import com.telecomscloud.ProgressRequestBody;
import com.telecomscloud.ProgressResponseBody;

import com.google.gson.reflect.TypeToken;

import com.squareup.okhttp.Call;
import com.squareup.okhttp.Interceptor;
import com.squareup.okhttp.Response;

import java.io.IOException;

import com.telecomscloud.model.AccountInfoResponseModel;
import com.telecomscloud.model.ClientGrantResponseModel;
import com.telecomscloud.model.CreateGrantClient;
import com.telecomscloud.model.FaxInfoCollectionResponseModel;
import com.telecomscloud.model.FaxOutboundSendResponseModel;
import java.io.File;
import com.telecomscloud.model.FaxOutboundGetUpdatesCollectionResponseModel;
import com.telecomscloud.model.FaxOutboundGetStatusResponseModel;
import com.telecomscloud.model.NumberFormatterResponseModel;
import com.telecomscloud.model.LocalityInfoResponseModel;
import com.telecomscloud.model.SmsOutboundSendResponseModel;
import com.telecomscloud.model.SmsOutboundGetStatusResponseModel;

import java.lang.reflect.Type;
import java.util.*;

public class VApi {
  private ApiClient apiClient;

  public VApi() {
    this(Configuration.getDefaultApiClient());
  }

  public VApi(ApiClient apiClient) {
    this.apiClient = apiClient;
  }

  public ApiClient getApiClient() {
    return apiClient;
  }

  public void setApiClient(ApiClient apiClient) {
    this.apiClient = apiClient;
  }

  
  /* Build call for accountInfo */
  private Call accountInfoCall(final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    

    // create path and map variables
    String path = "/v1/account/info".replaceAll("\\{format\\}","json");

    List<Pair> queryParams = new ArrayList<Pair>();

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "GET", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * Account Info
   * Provides account information
   * @return AccountInfoResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public AccountInfoResponseModel accountInfo() throws ApiException {
    ApiResponse<AccountInfoResponseModel> resp = accountInfoWithHttpInfo();
    return resp.getData();
  }

  /**
   * Account Info
   * Provides account information
   * @return ApiResponse<AccountInfoResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<AccountInfoResponseModel> accountInfoWithHttpInfo() throws ApiException {
    Call call = accountInfoCall(null, null);
    Type returnType = new TypeToken<AccountInfoResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   * Account Info (asynchronously)
   * Provides account information
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call accountInfoAsync(final ApiCallback<AccountInfoResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = accountInfoCall(progressListener, progressRequestListener);
    Type returnType = new TypeToken<AccountInfoResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
  /* Build call for createGrantClient */
  private Call createGrantClientCall(CreateGrantClient createGrantClient, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = createGrantClient;
    
    // verify the required parameter 'createGrantClient' is set
    if (createGrantClient == null) {
       throw new ApiException("Missing the required parameter 'createGrantClient' when calling createGrantClient(Async)");
    }
    

    // create path and map variables
    String path = "/v1/authorization/oauth2/grant-client".replaceAll("\\{format\\}","json");

    List<Pair> queryParams = new ArrayList<Pair>();

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      "application/json"
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] {  };
    return apiClient.buildCall(path, "POST", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * Request an access token using client credentials
   * Request an access token using client credentials. The returned access token can be used to make authenticated requests to any endpoint.
   * @param createGrantClient client_id : string - Unique client ID as issued by service provider. May be passed in query, body or along with client_secret in a basic auth header. client_secret : string - Client secret as issued by service provider. May be passed in json body or along with client_id in a basic auth header.
   * @return ClientGrantResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ClientGrantResponseModel createGrantClient(CreateGrantClient createGrantClient) throws ApiException {
    ApiResponse<ClientGrantResponseModel> resp = createGrantClientWithHttpInfo(createGrantClient);
    return resp.getData();
  }

  /**
   * Request an access token using client credentials
   * Request an access token using client credentials. The returned access token can be used to make authenticated requests to any endpoint.
   * @param createGrantClient client_id : string - Unique client ID as issued by service provider. May be passed in query, body or along with client_secret in a basic auth header. client_secret : string - Client secret as issued by service provider. May be passed in json body or along with client_id in a basic auth header.
   * @return ApiResponse<ClientGrantResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<ClientGrantResponseModel> createGrantClientWithHttpInfo(CreateGrantClient createGrantClient) throws ApiException {
    Call call = createGrantClientCall(createGrantClient, null, null);
    Type returnType = new TypeToken<ClientGrantResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   * Request an access token using client credentials (asynchronously)
   * Request an access token using client credentials. The returned access token can be used to make authenticated requests to any endpoint.
   * @param createGrantClient client_id : string - Unique client ID as issued by service provider. May be passed in query, body or along with client_secret in a basic auth header. client_secret : string - Client secret as issued by service provider. May be passed in json body or along with client_id in a basic auth header.
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call createGrantClientAsync(CreateGrantClient createGrantClient, final ApiCallback<ClientGrantResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = createGrantClientCall(createGrantClient, progressListener, progressRequestListener);
    Type returnType = new TypeToken<ClientGrantResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
  /* Build call for retrieveInfo */
  private Call retrieveInfoCall(String lastPointer, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    
    // verify the required parameter 'lastPointer' is set
    if (lastPointer == null) {
       throw new ApiException("Missing the required parameter 'lastPointer' when calling retrieveInfo(Async)");
    }
    

    // create path and map variables
    String path = "/v1/fax/inbound/info".replaceAll("\\{format\\}","json");

    List<Pair> queryParams = new ArrayList<Pair>();
    if (lastPointer != null)
      queryParams.addAll(apiClient.parameterToPairs("", "last_pointer", lastPointer));

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "GET", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * Received Fax Info
   * Provides received fax information including the faxes Id which can be used to download the fax document and a &#39;pointer&#39;. Make an initial request with the last_pointer parameter set to 0 (zero), this will return any items received within the last 48 hours, or an empty array if no items have been received. Each item record returned will include a &#39;pointer&#39;, to get updates received after the last check, pass the highest pointer seen.
   * @param lastPointer Latest known pointer, faxes records received after this will be returned. Enter 0 if unknown or no previous pointer..
   * @return FaxInfoCollectionResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public FaxInfoCollectionResponseModel retrieveInfo(String lastPointer) throws ApiException {
    ApiResponse<FaxInfoCollectionResponseModel> resp = retrieveInfoWithHttpInfo(lastPointer);
    return resp.getData();
  }

  /**
   * Received Fax Info
   * Provides received fax information including the faxes Id which can be used to download the fax document and a &#39;pointer&#39;. Make an initial request with the last_pointer parameter set to 0 (zero), this will return any items received within the last 48 hours, or an empty array if no items have been received. Each item record returned will include a &#39;pointer&#39;, to get updates received after the last check, pass the highest pointer seen.
   * @param lastPointer Latest known pointer, faxes records received after this will be returned. Enter 0 if unknown or no previous pointer..
   * @return ApiResponse<FaxInfoCollectionResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<FaxInfoCollectionResponseModel> retrieveInfoWithHttpInfo(String lastPointer) throws ApiException {
    Call call = retrieveInfoCall(lastPointer, null, null);
    Type returnType = new TypeToken<FaxInfoCollectionResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   * Received Fax Info (asynchronously)
   * Provides received fax information including the faxes Id which can be used to download the fax document and a &#39;pointer&#39;. Make an initial request with the last_pointer parameter set to 0 (zero), this will return any items received within the last 48 hours, or an empty array if no items have been received. Each item record returned will include a &#39;pointer&#39;, to get updates received after the last check, pass the highest pointer seen.
   * @param lastPointer Latest known pointer, faxes records received after this will be returned. Enter 0 if unknown or no previous pointer..
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call retrieveInfoAsync(String lastPointer, final ApiCallback<FaxInfoCollectionResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = retrieveInfoCall(lastPointer, progressListener, progressRequestListener);
    Type returnType = new TypeToken<FaxInfoCollectionResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
  /* Build call for retrieveFax */
  private Call retrieveFaxCall(String id, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    
    // verify the required parameter 'id' is set
    if (id == null) {
       throw new ApiException("Missing the required parameter 'id' when calling retrieveFax(Async)");
    }
    

    // create path and map variables
    String path = "/v1/fax/inbound/{id}".replaceAll("\\{format\\}","json")
      .replaceAll("\\{" + "id" + "\\}", apiClient.escapeString(id.toString()));

    List<Pair> queryParams = new ArrayList<Pair>();

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "GET", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * 
   * Retrieve fax
   * @param id ID
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public void retrieveFax(String id) throws ApiException {
    retrieveFaxWithHttpInfo(id);
  }

  /**
   * 
   * Retrieve fax
   * @param id ID
   * @return ApiResponse<Void>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<Void> retrieveFaxWithHttpInfo(String id) throws ApiException {
    Call call = retrieveFaxCall(id, null, null);
    return apiClient.execute(call);
  }

  /**
   *  (asynchronously)
   * Retrieve fax
   * @param id ID
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call retrieveFaxAsync(String id, final ApiCallback<Void> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = retrieveFaxCall(id, progressListener, progressRequestListener);
    apiClient.executeAsync(call, callback);
    return call;
  }
  
  /* Build call for sendFax */
  private Call sendFaxCall(String to, String from, File faxDocument, String numberFormatLocation, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    
    // verify the required parameter 'to' is set
    if (to == null) {
       throw new ApiException("Missing the required parameter 'to' when calling sendFax(Async)");
    }
    
    // verify the required parameter 'from' is set
    if (from == null) {
       throw new ApiException("Missing the required parameter 'from' when calling sendFax(Async)");
    }
    
    // verify the required parameter 'faxDocument' is set
    if (faxDocument == null) {
       throw new ApiException("Missing the required parameter 'faxDocument' when calling sendFax(Async)");
    }
    

    // create path and map variables
    String path = "/v1/fax/outbound".replaceAll("\\{format\\}","json");

    List<Pair> queryParams = new ArrayList<Pair>();
    if (to != null)
      queryParams.addAll(apiClient.parameterToPairs("", "to", to));
    if (from != null)
      queryParams.addAll(apiClient.parameterToPairs("", "from", from));
    if (numberFormatLocation != null)
      queryParams.addAll(apiClient.parameterToPairs("", "numberFormatLocation", numberFormatLocation));

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();
    if (faxDocument != null)
      formParams.put("fax_document", faxDocument);

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      "multipart/form-data"
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "POST", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * Submit a fax for sending
   * Post a PDF file as fax_document. Returns unique id.
   * @param to Number to send fax to.
   * @param from Recipient will receive the message as sent from this number, accepts E.164 numbers (with leading &#39;+&#39;) or UK based number formatting by default.
   * @param faxDocument Single PDF containing page(s) to be faxed.
   * @param numberFormatLocation Optional; ISO 3166-1 alpha-2 Two letter country code matching the location matching the provided number format, defaults to GB.
   * @return FaxOutboundSendResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public FaxOutboundSendResponseModel sendFax(String to, String from, File faxDocument, String numberFormatLocation) throws ApiException {
    ApiResponse<FaxOutboundSendResponseModel> resp = sendFaxWithHttpInfo(to, from, faxDocument, numberFormatLocation);
    return resp.getData();
  }

  /**
   * Submit a fax for sending
   * Post a PDF file as fax_document. Returns unique id.
   * @param to Number to send fax to.
   * @param from Recipient will receive the message as sent from this number, accepts E.164 numbers (with leading &#39;+&#39;) or UK based number formatting by default.
   * @param faxDocument Single PDF containing page(s) to be faxed.
   * @param numberFormatLocation Optional; ISO 3166-1 alpha-2 Two letter country code matching the location matching the provided number format, defaults to GB.
   * @return ApiResponse<FaxOutboundSendResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<FaxOutboundSendResponseModel> sendFaxWithHttpInfo(String to, String from, File faxDocument, String numberFormatLocation) throws ApiException {
    Call call = sendFaxCall(to, from, faxDocument, numberFormatLocation, null, null);
    Type returnType = new TypeToken<FaxOutboundSendResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   * Submit a fax for sending (asynchronously)
   * Post a PDF file as fax_document. Returns unique id.
   * @param to Number to send fax to.
   * @param from Recipient will receive the message as sent from this number, accepts E.164 numbers (with leading &#39;+&#39;) or UK based number formatting by default.
   * @param faxDocument Single PDF containing page(s) to be faxed.
   * @param numberFormatLocation Optional; ISO 3166-1 alpha-2 Two letter country code matching the location matching the provided number format, defaults to GB.
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call sendFaxAsync(String to, String from, File faxDocument, String numberFormatLocation, final ApiCallback<FaxOutboundSendResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = sendFaxCall(to, from, faxDocument, numberFormatLocation, progressListener, progressRequestListener);
    Type returnType = new TypeToken<FaxOutboundSendResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
  /* Build call for retrieveFaxUpdates */
  private Call retrieveFaxUpdatesCall(String lastPointer, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    

    // create path and map variables
    String path = "/v1/fax/outbound/updates".replaceAll("\\{format\\}","json");

    List<Pair> queryParams = new ArrayList<Pair>();
    if (lastPointer != null)
      queryParams.addAll(apiClient.parameterToPairs("", "last_pointer", lastPointer));

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "GET", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * Get fax submission status updates
   * Retrieve any status updates for fax submissions since last checked.
   * @param lastPointer Optional pointer, if not used last two days will be returned.
   * @return FaxOutboundGetUpdatesCollectionResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public FaxOutboundGetUpdatesCollectionResponseModel retrieveFaxUpdates(String lastPointer) throws ApiException {
    ApiResponse<FaxOutboundGetUpdatesCollectionResponseModel> resp = retrieveFaxUpdatesWithHttpInfo(lastPointer);
    return resp.getData();
  }

  /**
   * Get fax submission status updates
   * Retrieve any status updates for fax submissions since last checked.
   * @param lastPointer Optional pointer, if not used last two days will be returned.
   * @return ApiResponse<FaxOutboundGetUpdatesCollectionResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<FaxOutboundGetUpdatesCollectionResponseModel> retrieveFaxUpdatesWithHttpInfo(String lastPointer) throws ApiException {
    Call call = retrieveFaxUpdatesCall(lastPointer, null, null);
    Type returnType = new TypeToken<FaxOutboundGetUpdatesCollectionResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   * Get fax submission status updates (asynchronously)
   * Retrieve any status updates for fax submissions since last checked.
   * @param lastPointer Optional pointer, if not used last two days will be returned.
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call retrieveFaxUpdatesAsync(String lastPointer, final ApiCallback<FaxOutboundGetUpdatesCollectionResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = retrieveFaxUpdatesCall(lastPointer, progressListener, progressRequestListener);
    Type returnType = new TypeToken<FaxOutboundGetUpdatesCollectionResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
  /* Build call for retrieveFaxStatus */
  private Call retrieveFaxStatusCall(String id, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    
    // verify the required parameter 'id' is set
    if (id == null) {
       throw new ApiException("Missing the required parameter 'id' when calling retrieveFaxStatus(Async)");
    }
    

    // create path and map variables
    String path = "/v1/fax/outbound/{id}/status".replaceAll("\\{format\\}","json")
      .replaceAll("\\{" + "id" + "\\}", apiClient.escapeString(id.toString()));

    List<Pair> queryParams = new ArrayList<Pair>();

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "GET", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * Check the status of a submitted fax.
   * Returns the status of the fax submission referenced by the passed Id
   * @param id Id of message to check as provided on submission of Fax.
   * @return FaxOutboundGetStatusResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public FaxOutboundGetStatusResponseModel retrieveFaxStatus(String id) throws ApiException {
    ApiResponse<FaxOutboundGetStatusResponseModel> resp = retrieveFaxStatusWithHttpInfo(id);
    return resp.getData();
  }

  /**
   * Check the status of a submitted fax.
   * Returns the status of the fax submission referenced by the passed Id
   * @param id Id of message to check as provided on submission of Fax.
   * @return ApiResponse<FaxOutboundGetStatusResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<FaxOutboundGetStatusResponseModel> retrieveFaxStatusWithHttpInfo(String id) throws ApiException {
    Call call = retrieveFaxStatusCall(id, null, null);
    Type returnType = new TypeToken<FaxOutboundGetStatusResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   * Check the status of a submitted fax. (asynchronously)
   * Returns the status of the fax submission referenced by the passed Id
   * @param id Id of message to check as provided on submission of Fax.
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call retrieveFaxStatusAsync(String id, final ApiCallback<FaxOutboundGetStatusResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = retrieveFaxStatusCall(id, progressListener, progressRequestListener);
    Type returnType = new TypeToken<FaxOutboundGetStatusResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
  /* Build call for formatE164 */
  private Call formatE164Call(String number, String location, String pretty, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    
    // verify the required parameter 'number' is set
    if (number == null) {
       throw new ApiException("Missing the required parameter 'number' when calling formatE164(Async)");
    }
    
    // verify the required parameter 'location' is set
    if (location == null) {
       throw new ApiException("Missing the required parameter 'location' when calling formatE164(Async)");
    }
    

    // create path and map variables
    String path = "/v1/format/number/e164/{number}/{location}".replaceAll("\\{format\\}","json")
      .replaceAll("\\{" + "number" + "\\}", apiClient.escapeString(number.toString()))
      .replaceAll("\\{" + "location" + "\\}", apiClient.escapeString(location.toString()));

    List<Pair> queryParams = new ArrayList<Pair>();
    if (pretty != null)
      queryParams.addAll(apiClient.parameterToPairs("", "pretty", pretty));

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "GET", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * Get E.164 format number
   * Provides an E.164 formatted (e.g. +441514962245) version of the provided national (e.g. 01514962245) format phone number
   * @param number National format phone number (e.g. 01514962245).
   * @param location Two letter country code matching the location the provided number is formatted for (e.g. US).
   * @param pretty Optional query parameter. If true, returned number will contain whitespace for readability (e.g. +44 151 496 2245).
   * @return NumberFormatterResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public NumberFormatterResponseModel formatE164(String number, String location, String pretty) throws ApiException {
    ApiResponse<NumberFormatterResponseModel> resp = formatE164WithHttpInfo(number, location, pretty);
    return resp.getData();
  }

  /**
   * Get E.164 format number
   * Provides an E.164 formatted (e.g. +441514962245) version of the provided national (e.g. 01514962245) format phone number
   * @param number National format phone number (e.g. 01514962245).
   * @param location Two letter country code matching the location the provided number is formatted for (e.g. US).
   * @param pretty Optional query parameter. If true, returned number will contain whitespace for readability (e.g. +44 151 496 2245).
   * @return ApiResponse<NumberFormatterResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<NumberFormatterResponseModel> formatE164WithHttpInfo(String number, String location, String pretty) throws ApiException {
    Call call = formatE164Call(number, location, pretty, null, null);
    Type returnType = new TypeToken<NumberFormatterResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   * Get E.164 format number (asynchronously)
   * Provides an E.164 formatted (e.g. +441514962245) version of the provided national (e.g. 01514962245) format phone number
   * @param number National format phone number (e.g. 01514962245).
   * @param location Two letter country code matching the location the provided number is formatted for (e.g. US).
   * @param pretty Optional query parameter. If true, returned number will contain whitespace for readability (e.g. +44 151 496 2245).
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call formatE164Async(String number, String location, String pretty, final ApiCallback<NumberFormatterResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = formatE164Call(number, location, pretty, progressListener, progressRequestListener);
    Type returnType = new TypeToken<NumberFormatterResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
  /* Build call for formatInternational */
  private Call formatInternationalCall(String number, String providedLocation, String dialFromLocation, String pretty, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    
    // verify the required parameter 'number' is set
    if (number == null) {
       throw new ApiException("Missing the required parameter 'number' when calling formatInternational(Async)");
    }
    
    // verify the required parameter 'providedLocation' is set
    if (providedLocation == null) {
       throw new ApiException("Missing the required parameter 'providedLocation' when calling formatInternational(Async)");
    }
    
    // verify the required parameter 'dialFromLocation' is set
    if (dialFromLocation == null) {
       throw new ApiException("Missing the required parameter 'dialFromLocation' when calling formatInternational(Async)");
    }
    

    // create path and map variables
    String path = "/v1/format/number/international/{number}/{providedLocation}/{dialFromLocation}".replaceAll("\\{format\\}","json")
      .replaceAll("\\{" + "number" + "\\}", apiClient.escapeString(number.toString()))
      .replaceAll("\\{" + "providedLocation" + "\\}", apiClient.escapeString(providedLocation.toString()))
      .replaceAll("\\{" + "dialFromLocation" + "\\}", apiClient.escapeString(dialFromLocation.toString()));

    List<Pair> queryParams = new ArrayList<Pair>();
    if (pretty != null)
      queryParams.addAll(apiClient.parameterToPairs("", "pretty", pretty));

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "GET", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * Get international format number
   * Converts the provided National format number (e.g. 01514962245) into an dialable number in the provided location (e.g. 011441514962245 for dialing GB from US)
   * @param number Phone number in National format (e.g. 01514962245).
   * @param providedLocation Two letter country code matching provided number format (e.g. GB).
   * @param dialFromLocation Two letter country code of location dialling from (e.g. US).
   * @param pretty Optional query parameter. If true, returned number will contain whitespace for readability (e.g. 011 44 151 496 2245).
   * @return NumberFormatterResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public NumberFormatterResponseModel formatInternational(String number, String providedLocation, String dialFromLocation, String pretty) throws ApiException {
    ApiResponse<NumberFormatterResponseModel> resp = formatInternationalWithHttpInfo(number, providedLocation, dialFromLocation, pretty);
    return resp.getData();
  }

  /**
   * Get international format number
   * Converts the provided National format number (e.g. 01514962245) into an dialable number in the provided location (e.g. 011441514962245 for dialing GB from US)
   * @param number Phone number in National format (e.g. 01514962245).
   * @param providedLocation Two letter country code matching provided number format (e.g. GB).
   * @param dialFromLocation Two letter country code of location dialling from (e.g. US).
   * @param pretty Optional query parameter. If true, returned number will contain whitespace for readability (e.g. 011 44 151 496 2245).
   * @return ApiResponse<NumberFormatterResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<NumberFormatterResponseModel> formatInternationalWithHttpInfo(String number, String providedLocation, String dialFromLocation, String pretty) throws ApiException {
    Call call = formatInternationalCall(number, providedLocation, dialFromLocation, pretty, null, null);
    Type returnType = new TypeToken<NumberFormatterResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   * Get international format number (asynchronously)
   * Converts the provided National format number (e.g. 01514962245) into an dialable number in the provided location (e.g. 011441514962245 for dialing GB from US)
   * @param number Phone number in National format (e.g. 01514962245).
   * @param providedLocation Two letter country code matching provided number format (e.g. GB).
   * @param dialFromLocation Two letter country code of location dialling from (e.g. US).
   * @param pretty Optional query parameter. If true, returned number will contain whitespace for readability (e.g. 011 44 151 496 2245).
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call formatInternationalAsync(String number, String providedLocation, String dialFromLocation, String pretty, final ApiCallback<NumberFormatterResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = formatInternationalCall(number, providedLocation, dialFromLocation, pretty, progressListener, progressRequestListener);
    Type returnType = new TypeToken<NumberFormatterResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
  /* Build call for formatNational */
  private Call formatNationalCall(String number, String location, String pretty, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    
    // verify the required parameter 'number' is set
    if (number == null) {
       throw new ApiException("Missing the required parameter 'number' when calling formatNational(Async)");
    }
    
    // verify the required parameter 'location' is set
    if (location == null) {
       throw new ApiException("Missing the required parameter 'location' when calling formatNational(Async)");
    }
    

    // create path and map variables
    String path = "/v1/format/number/national/{number}/{location}".replaceAll("\\{format\\}","json")
      .replaceAll("\\{" + "number" + "\\}", apiClient.escapeString(number.toString()))
      .replaceAll("\\{" + "location" + "\\}", apiClient.escapeString(location.toString()));

    List<Pair> queryParams = new ArrayList<Pair>();
    if (pretty != null)
      queryParams.addAll(apiClient.parameterToPairs("", "pretty", pretty));

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "GET", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * Get national format number
   * Provides the national (e.g. 01514962245) format variant of the provided E.164 (e.g. +441514962245)
   * @param number Phone number in E.164 format (e.g. +441514962245).
   * @param location Two letter country code (e.g. US).
   * @param pretty Optional query parameter. If true, returned number will contain whitespace for readability (e.g. 0151 496 2245).
   * @return NumberFormatterResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public NumberFormatterResponseModel formatNational(String number, String location, String pretty) throws ApiException {
    ApiResponse<NumberFormatterResponseModel> resp = formatNationalWithHttpInfo(number, location, pretty);
    return resp.getData();
  }

  /**
   * Get national format number
   * Provides the national (e.g. 01514962245) format variant of the provided E.164 (e.g. +441514962245)
   * @param number Phone number in E.164 format (e.g. +441514962245).
   * @param location Two letter country code (e.g. US).
   * @param pretty Optional query parameter. If true, returned number will contain whitespace for readability (e.g. 0151 496 2245).
   * @return ApiResponse<NumberFormatterResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<NumberFormatterResponseModel> formatNationalWithHttpInfo(String number, String location, String pretty) throws ApiException {
    Call call = formatNationalCall(number, location, pretty, null, null);
    Type returnType = new TypeToken<NumberFormatterResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   * Get national format number (asynchronously)
   * Provides the national (e.g. 01514962245) format variant of the provided E.164 (e.g. +441514962245)
   * @param number Phone number in E.164 format (e.g. +441514962245).
   * @param location Two letter country code (e.g. US).
   * @param pretty Optional query parameter. If true, returned number will contain whitespace for readability (e.g. 0151 496 2245).
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call formatNationalAsync(String number, String location, String pretty, final ApiCallback<NumberFormatterResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = formatNationalCall(number, location, pretty, progressListener, progressRequestListener);
    Type returnType = new TypeToken<NumberFormatterResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
  /* Build call for formatPretty */
  private Call formatPrettyCall(String number, String location, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    
    // verify the required parameter 'number' is set
    if (number == null) {
       throw new ApiException("Missing the required parameter 'number' when calling formatPretty(Async)");
    }
    
    // verify the required parameter 'location' is set
    if (location == null) {
       throw new ApiException("Missing the required parameter 'location' when calling formatPretty(Async)");
    }
    

    // create path and map variables
    String path = "/v1/format/number/pretty/{number}/{location}".replaceAll("\\{format\\}","json")
      .replaceAll("\\{" + "number" + "\\}", apiClient.escapeString(number.toString()))
      .replaceAll("\\{" + "location" + "\\}", apiClient.escapeString(location.toString()));

    List<Pair> queryParams = new ArrayList<Pair>();

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "GET", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * Get pretty number
   * Uses whitespace to provide human readable numbers
   * @param number Phone number to format (e.g. 01514962245, 011441514962245, +441514962245).
   * @param location Two letter country code matching provided number format (e.g. GB).
   * @return NumberFormatterResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public NumberFormatterResponseModel formatPretty(String number, String location) throws ApiException {
    ApiResponse<NumberFormatterResponseModel> resp = formatPrettyWithHttpInfo(number, location);
    return resp.getData();
  }

  /**
   * Get pretty number
   * Uses whitespace to provide human readable numbers
   * @param number Phone number to format (e.g. 01514962245, 011441514962245, +441514962245).
   * @param location Two letter country code matching provided number format (e.g. GB).
   * @return ApiResponse<NumberFormatterResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<NumberFormatterResponseModel> formatPrettyWithHttpInfo(String number, String location) throws ApiException {
    Call call = formatPrettyCall(number, location, null, null);
    Type returnType = new TypeToken<NumberFormatterResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   * Get pretty number (asynchronously)
   * Uses whitespace to provide human readable numbers
   * @param number Phone number to format (e.g. 01514962245, 011441514962245, +441514962245).
   * @param location Two letter country code matching provided number format (e.g. GB).
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call formatPrettyAsync(String number, String location, final ApiCallback<NumberFormatterResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = formatPrettyCall(number, location, progressListener, progressRequestListener);
    Type returnType = new TypeToken<NumberFormatterResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
  /* Build call for retrieveLocalityInfo */
  private Call retrieveLocalityInfoCall(String phoneNumber, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    
    // verify the required parameter 'phoneNumber' is set
    if (phoneNumber == null) {
       throw new ApiException("Missing the required parameter 'phoneNumber' when calling retrieveLocalityInfo(Async)");
    }
    

    // create path and map variables
    String path = "/v1/locality-lookup/{phoneNumber}".replaceAll("\\{format\\}","json")
      .replaceAll("\\{" + "phoneNumber" + "\\}", apiClient.escapeString(phoneNumber.toString()));

    List<Pair> queryParams = new ArrayList<Pair>();

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "GET", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * 
   * Retrieve locality info
   * @param phoneNumber Phone number
   * @return LocalityInfoResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public LocalityInfoResponseModel retrieveLocalityInfo(String phoneNumber) throws ApiException {
    ApiResponse<LocalityInfoResponseModel> resp = retrieveLocalityInfoWithHttpInfo(phoneNumber);
    return resp.getData();
  }

  /**
   * 
   * Retrieve locality info
   * @param phoneNumber Phone number
   * @return ApiResponse<LocalityInfoResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<LocalityInfoResponseModel> retrieveLocalityInfoWithHttpInfo(String phoneNumber) throws ApiException {
    Call call = retrieveLocalityInfoCall(phoneNumber, null, null);
    Type returnType = new TypeToken<LocalityInfoResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   *  (asynchronously)
   * Retrieve locality info
   * @param phoneNumber Phone number
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call retrieveLocalityInfoAsync(String phoneNumber, final ApiCallback<LocalityInfoResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = retrieveLocalityInfoCall(phoneNumber, progressListener, progressRequestListener);
    Type returnType = new TypeToken<LocalityInfoResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
  /* Build call for retrieveTps */
  private Call retrieveTpsCall(String phoneNumber, String numberFormatLocation, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    
    // verify the required parameter 'phoneNumber' is set
    if (phoneNumber == null) {
       throw new ApiException("Missing the required parameter 'phoneNumber' when calling retrieveTps(Async)");
    }
    

    // create path and map variables
    String path = "/v1/recipient-preferences/tps/{phone_number}".replaceAll("\\{format\\}","json")
      .replaceAll("\\{" + "phone_number" + "\\}", apiClient.escapeString(phoneNumber.toString()));

    List<Pair> queryParams = new ArrayList<Pair>();
    if (numberFormatLocation != null)
      queryParams.addAll(apiClient.parameterToPairs("", "numberFormatLocation", numberFormatLocation));

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "GET", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * Check Telephone Preference Service
   * Check if provided number is included in the Telephony Preference Service list. Returns true or false.
   * @param phoneNumber Phone number to lookup in E.164 format or national format according to numberFormatLocation.
   * @param numberFormatLocation Optional; ISO 3166-1 alpha-2 Two letter country code matching the origin of the provided number format, defaults to GB.
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public void retrieveTps(String phoneNumber, String numberFormatLocation) throws ApiException {
    retrieveTpsWithHttpInfo(phoneNumber, numberFormatLocation);
  }

  /**
   * Check Telephone Preference Service
   * Check if provided number is included in the Telephony Preference Service list. Returns true or false.
   * @param phoneNumber Phone number to lookup in E.164 format or national format according to numberFormatLocation.
   * @param numberFormatLocation Optional; ISO 3166-1 alpha-2 Two letter country code matching the origin of the provided number format, defaults to GB.
   * @return ApiResponse<Void>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<Void> retrieveTpsWithHttpInfo(String phoneNumber, String numberFormatLocation) throws ApiException {
    Call call = retrieveTpsCall(phoneNumber, numberFormatLocation, null, null);
    return apiClient.execute(call);
  }

  /**
   * Check Telephone Preference Service (asynchronously)
   * Check if provided number is included in the Telephony Preference Service list. Returns true or false.
   * @param phoneNumber Phone number to lookup in E.164 format or national format according to numberFormatLocation.
   * @param numberFormatLocation Optional; ISO 3166-1 alpha-2 Two letter country code matching the origin of the provided number format, defaults to GB.
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call retrieveTpsAsync(String phoneNumber, String numberFormatLocation, final ApiCallback<Void> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = retrieveTpsCall(phoneNumber, numberFormatLocation, progressListener, progressRequestListener);
    apiClient.executeAsync(call, callback);
    return call;
  }
  
  /* Build call for sendSms */
  private Call sendSmsCall(SmsOutboundSendResponseModel smsOutboundSendResponseModel, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = smsOutboundSendResponseModel;
    
    // verify the required parameter 'smsOutboundSendResponseModel' is set
    if (smsOutboundSendResponseModel == null) {
       throw new ApiException("Missing the required parameter 'smsOutboundSendResponseModel' when calling sendSms(Async)");
    }
    

    // create path and map variables
    String path = "/v1/sms/outbound".replaceAll("\\{format\\}","json");

    List<Pair> queryParams = new ArrayList<Pair>();

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      "application/json"
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "POST", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * Send an SMS Text Message
   * Send an SMS message from one of your TC numbers, or the default number (this can be found on the API Access settings on the TelecomsCloud website). SMS message is queued for delivery to the specified recipient, returning a unique reference on success. This reference can be used to check the status of the submitted message.
   * @param smsOutboundSendResponseModel * to : string (required) - Number to send message to.  * from : string (required) - Recipient will receive the message as sent from this number, accepts E.164 numbers (with leading &#39;+&#39;) or UK based number formatting by default.  * message : string (required) - Message to send to recipient, up to 160 chars.  * numberFormatLocation : string - Optional; ISO 3166-1 alpha-2 Two letter country code matching the location matching the provided number format, defaults to GB.
   * @return SmsOutboundSendResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public SmsOutboundSendResponseModel sendSms(SmsOutboundSendResponseModel smsOutboundSendResponseModel) throws ApiException {
    ApiResponse<SmsOutboundSendResponseModel> resp = sendSmsWithHttpInfo(smsOutboundSendResponseModel);
    return resp.getData();
  }

  /**
   * Send an SMS Text Message
   * Send an SMS message from one of your TC numbers, or the default number (this can be found on the API Access settings on the TelecomsCloud website). SMS message is queued for delivery to the specified recipient, returning a unique reference on success. This reference can be used to check the status of the submitted message.
   * @param smsOutboundSendResponseModel * to : string (required) - Number to send message to.  * from : string (required) - Recipient will receive the message as sent from this number, accepts E.164 numbers (with leading &#39;+&#39;) or UK based number formatting by default.  * message : string (required) - Message to send to recipient, up to 160 chars.  * numberFormatLocation : string - Optional; ISO 3166-1 alpha-2 Two letter country code matching the location matching the provided number format, defaults to GB.
   * @return ApiResponse<SmsOutboundSendResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<SmsOutboundSendResponseModel> sendSmsWithHttpInfo(SmsOutboundSendResponseModel smsOutboundSendResponseModel) throws ApiException {
    Call call = sendSmsCall(smsOutboundSendResponseModel, null, null);
    Type returnType = new TypeToken<SmsOutboundSendResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   * Send an SMS Text Message (asynchronously)
   * Send an SMS message from one of your TC numbers, or the default number (this can be found on the API Access settings on the TelecomsCloud website). SMS message is queued for delivery to the specified recipient, returning a unique reference on success. This reference can be used to check the status of the submitted message.
   * @param smsOutboundSendResponseModel * to : string (required) - Number to send message to.  * from : string (required) - Recipient will receive the message as sent from this number, accepts E.164 numbers (with leading &#39;+&#39;) or UK based number formatting by default.  * message : string (required) - Message to send to recipient, up to 160 chars.  * numberFormatLocation : string - Optional; ISO 3166-1 alpha-2 Two letter country code matching the location matching the provided number format, defaults to GB.
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call sendSmsAsync(SmsOutboundSendResponseModel smsOutboundSendResponseModel, final ApiCallback<SmsOutboundSendResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = sendSmsCall(smsOutboundSendResponseModel, progressListener, progressRequestListener);
    Type returnType = new TypeToken<SmsOutboundSendResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
  /* Build call for getSmsStatus */
  private Call getSmsStatusCall(String smsId, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
    Object postBody = null;
    
    // verify the required parameter 'smsId' is set
    if (smsId == null) {
       throw new ApiException("Missing the required parameter 'smsId' when calling getSmsStatus(Async)");
    }
    

    // create path and map variables
    String path = "/v1/sms/outbound/{sms_id}/status".replaceAll("\\{format\\}","json")
      .replaceAll("\\{" + "sms_id" + "\\}", apiClient.escapeString(smsId.toString()));

    List<Pair> queryParams = new ArrayList<Pair>();

    Map<String, String> headerParams = new HashMap<String, String>();

    Map<String, Object> formParams = new HashMap<String, Object>();

    final String[] accepts = {
      "application/json"
    };
    final String accept = apiClient.selectHeaderAccept(accepts);
    if (accept != null) headerParams.put("Accept", accept);

    final String[] contentTypes = {
      
    };
    final String contentType = apiClient.selectHeaderContentType(contentTypes);
    headerParams.put("Content-Type", contentType);

    if(progressListener != null) {
      apiClient.getHttpClient().networkInterceptors().add(new Interceptor() {
      @Override
      public Response intercept(Interceptor.Chain chain) throws IOException {
        Response originalResponse = chain.proceed(chain.request());
        return originalResponse.newBuilder()
                .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                .build();
        }
      });
    }

    String[] authNames = new String[] { "Default" };
    return apiClient.buildCall(path, "GET", queryParams, postBody, headerParams, formParams, authNames, progressRequestListener);
  }

  /**
   * 
   * Get sms status
   * @param smsId Sms ID
   * @return SmsOutboundGetStatusResponseModel
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public SmsOutboundGetStatusResponseModel getSmsStatus(String smsId) throws ApiException {
    ApiResponse<SmsOutboundGetStatusResponseModel> resp = getSmsStatusWithHttpInfo(smsId);
    return resp.getData();
  }

  /**
   * 
   * Get sms status
   * @param smsId Sms ID
   * @return ApiResponse<SmsOutboundGetStatusResponseModel>
   * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
   */
  public ApiResponse<SmsOutboundGetStatusResponseModel> getSmsStatusWithHttpInfo(String smsId) throws ApiException {
    Call call = getSmsStatusCall(smsId, null, null);
    Type returnType = new TypeToken<SmsOutboundGetStatusResponseModel>(){}.getType();
    return apiClient.execute(call, returnType);
  }

  /**
   *  (asynchronously)
   * Get sms status
   * @param smsId Sms ID
   * @param callback The callback to be executed when the API call finishes
   * @return The request call
   * @throws ApiException If fail to process the API call, e.g. serializing the request body object
   */
  public Call getSmsStatusAsync(String smsId, final ApiCallback<SmsOutboundGetStatusResponseModel> callback) throws ApiException {

    ProgressResponseBody.ProgressListener progressListener = null;
    ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

    if (callback != null) {
      progressListener = new ProgressResponseBody.ProgressListener() {
        @Override
        public void update(long bytesRead, long contentLength, boolean done) {
          callback.onDownloadProgress(bytesRead, contentLength, done);
        } 
      };

      progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
        @Override
        public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
          callback.onUploadProgress(bytesWritten, contentLength, done);
        }
      };
    }

    Call call = getSmsStatusCall(smsId, progressListener, progressRequestListener);
    Type returnType = new TypeToken<SmsOutboundGetStatusResponseModel>(){}.getType();
    apiClient.executeAsync(call, returnType, callback);
    return call;
  }
  
}
